@@ -1,5 +1,6 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
+import datetime
 from decimal import Decimal
 from itertools import groupby, chain
 from functools import partial
@@ -762,19 +763,17 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
             self.raise_user_error('missing_account_receivable',
                 (self.party.rec_name,))
 
-        invoice_lines = {}
+        invoice_lines = []
         for line in self.lines:
-            ilines = line.get_invoice_line()
-            if ilines:
-                invoice_lines[line.id] = ilines
+            invoice_lines.append(line.get_invoice_line())
+        invoice_lines = list(chain(*invoice_lines))
         if not invoice_lines:
             return
 
         invoice = self._get_invoice_sale()
-        invoice.lines = ((list(invoice.lines)
-                if hasattr(invoice, 'lines') else [])
-            + list(chain.from_iterable(invoice_lines[l.id] for l in self.lines
-                    if l.id in invoice_lines)))
+        if getattr(invoice, 'lines', None):
+            invoice_lines = list(invoice.lines) + invoice_lines
+        invoice.lines = invoice_lines
         invoice.save()
 
         Invoice.update_taxes([invoice])
@@ -790,7 +789,10 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
         line_id, move = move
         line = SaleLine(line_id)
 
-        planned_date = max(m.planned_date for m in moves)
+        if any(m.planned_date is None for m in moves):
+            planned_date = None
+        else:
+            planned_date = max(m.planned_date for m in moves)
         return (
             ('planned_date', planned_date),
             ('warehouse', line.warehouse.id),
@@ -1006,6 +1008,13 @@ class SaleLine(sequence_ordered(), ModelSQL, ModelView):
                 [Eval('_parent_sale', {}).get('warehouse', 0)], []),
             'stock_date_end': Eval('_parent_sale', {}).get('sale_date'),
             'stock_skip_warehouse': True,
+            # From _get_context_sale_price
+            'currency': Eval('_parent_sale', {}).get('currency'),
+            'customer': Eval('_parent_sale', {}).get('party'),
+            'sale_date': Eval('_parent_sale', {}).get('sale_date'),
+            'uom': Eval('unit'),
+            'taxes': Eval('taxes', []),
+            'quantity': Eval('quantity'),
             }, depends=['type', 'sale_state'])
     product_uom_category = fields.Function(
         fields.Many2One('product.uom.category', 'Product Uom Category'),
@@ -1074,8 +1083,8 @@ class SaleLine(sequence_ordered(), ModelSQL, ModelView):
                     'customer location in line "%(line)s".'),
                 'missing_account_revenue': ('Product "%(product)s" of sale '
                     '%(sale)s misses a revenue account.'),
-                'missing_account_revenue_property': ('Sale "%(sale)s" '
-                    'misses an "account revenue" default property.'),
+                'missing_default_account_revenue': ('Sale "%(sale)s" '
+                    'misses a default "account revenue".'),
                 })
 
     @classmethod
@@ -1110,26 +1119,28 @@ class SaleLine(sequence_ordered(), ModelSQL, ModelView):
             return self.unit.digits
         return 2
 
-    def get_move_done(self, name):
-        Uom = Pool().get('product.uom')
-        done = True
+    @property
+    def _move_remaining_quantity(self):
+        "Compute the remaining quantity to ship"
+        pool = Pool()
+        Uom = pool.get('product.uom')
         if self.type != 'line' or not self.product:
-            return True
+            return
         if self.product.type == 'service':
-            return True
+            return
         skip_ids = set(x.id for x in self.moves_ignored)
-        skip_ids.update(x.id for x in self.moves_recreated)
-        quantity = self.quantity
+        quantity = abs(self.quantity)
         for move in self.moves:
-            if move.state != 'done' \
-                    and move.id not in skip_ids:
-                done = False
-                break
-            quantity -= Uom.compute_qty(move.uom, move.quantity, self.unit)
-        if done:
-            if self.unit.round(quantity) > 0.0:
-                done = False
-        return done
+            if move.state == 'done' or move.id in skip_ids:
+                quantity -= Uom.compute_qty(move.uom, move.quantity, self.unit)
+        return quantity
+
+    def get_move_done(self, name):
+        quantity = self._move_remaining_quantity
+        if quantity is None:
+            return True
+        else:
+            return self.unit.round(quantity) <= 0
 
     def get_move_exception(self, name):
         skip_ids = set(x.id for x in self.moves_ignored)
@@ -1279,11 +1290,13 @@ class SaleLine(sequence_ordered(), ModelSQL, ModelView):
             if self.warehouse:
                 return self.warehouse.output_location.id
         else:
-            return self.sale.party.customer_location.id
+            party = self.sale.shipment_party or self.sale.party
+            return party.customer_location.id
 
     def get_to_location(self, name):
         if (self.quantity or 0) >= 0:
-            return self.sale.party.customer_location.id
+            party = self.sale.shipment_party or self.sale.party
+            return party.customer_location.id
         else:
             if self.warehouse:
                 return self.warehouse.input_location.id
@@ -1299,7 +1312,10 @@ class SaleLine(sequence_ordered(), ModelSQL, ModelView):
                 return
         if self.product and self.quantity is not None and self.quantity > 0:
             date = self.sale.sale_date if self.sale else None
-            return self.product.compute_shipping_date(date=date)
+            shipping_date = self.product.compute_shipping_date(date=date)
+            if shipping_date == datetime.date.max:
+                return None
+            return shipping_date
 
     @fields.depends('sale', '_parent_sale.state')
     def on_change_with_sale_state(self, name=None):
@@ -1309,8 +1325,9 @@ class SaleLine(sequence_ordered(), ModelSQL, ModelView):
     def get_invoice_line(self):
         'Return a list of invoice lines for sale line'
         pool = Pool()
-        Property = pool.get('ir.property')
         InvoiceLine = pool.get('account.invoice.line')
+        AccountConfiguration = pool.get('account.configuration')
+        account_config = AccountConfiguration(1)
 
         invoice_line = InvoiceLine()
         invoice_line.type = self.type
@@ -1346,12 +1363,13 @@ class SaleLine(sequence_ordered(), ModelSQL, ModelView):
                         'product': self.product.rec_name,
                         })
         else:
-            for model in ('product.template', 'product.category'):
-                invoice_line.account = Property.get('account_revenue', model)
+            for name in ['default_product_account_revenue',
+                    'default_category_account_revenue']:
+                invoice_line.account = account_config.get_multivalue(name)
                 if invoice_line.account:
                     break
             if not invoice_line.account:
-                self.raise_user_error('missing_account_revenue_property', {
+                self.raise_user_error('missing_default_account_revenue', {
                         'sale': self.sale.rec_name,
                         })
         invoice_line.stock_moves = self._get_invoice_line_moves()
@@ -1701,13 +1719,9 @@ class HandleInvoiceException(Wizard):
 
         sale = Sale(Transaction().context['active_id'])
 
-        skips = set(sale.invoices_ignored)
-        skips.update(sale.invoices_recreated)
         invoices_ignored = []
         invoices_recreated = []
-        for invoice in sale.invoices:
-            if invoice not in self.ask.domain_invoices or invoice in skips:
-                continue
+        for invoice in self.ask.domain_invoices:
             if invoice in self.ask.recreate_invoices:
                 invoices_recreated.append(invoice.id)
             else:
