@@ -4,8 +4,7 @@ import unittest
 import doctest
 import trytond.tests.test_tryton
 from trytond.tests.test_tryton import ModuleTestCase
-from trytond.tests.test_tryton import (doctest_setup, doctest_teardown,
-    doctest_checker)
+from trytond.tests.test_tryton import doctest_teardown, doctest_checker
 
 
 class SaleTestCase(ModuleTestCase):
@@ -17,7 +16,7 @@ def suite():
     suite = trytond.tests.test_tryton.suite()
     suite.addTests(unittest.TestLoader().loadTestsFromTestCase(SaleTestCase))
     suite.addTests(doctest.DocFileSuite('scenario_sale.rst',
-            setUp=doctest_setup, tearDown=doctest_teardown, encoding='utf-8',
+            tearDown=doctest_teardown, encoding='utf-8',
             optionflags=doctest.REPORT_ONLY_FIRST_FAILURE,
             checker=doctest_checker))
     return suite
